c----------------------------------------------------------------------
C  couplings.f 
c----------------------------------------------------------------------
c  This files takes the inputs of the standard model from a Les Houches 
c  file (param_card.dat) and calculates all the couplings that end up
c  in the Feynman rules, i.e., in the HELAS routines.
c   
c  With the convention of the New Web Generation MadEvent in this
c  part no non-trivial computation is made. The SM model secondary
c  parameters have been all calculated by the SM calculator, SMCalc
c  which produced param_card.dat.
c
c  The only exception to the above rule is for alpha_S. In the case
c  where a pdf is used in the initial state, then the values as(MZ)
c  set in the les houches card is superseeded.
c  Scales and pdf's are set in the run_card.dat.
c
c This file contains the following routines:
c 
c- madgraph original call to set the parameters
c- lh_readin is called from here.
c  It talks to the lh_readin through the common block values.
c      subroutine setpara
c
c-routine to read the LH parameters in
c      subroutine lh_readin
c
c-to set
c      subroutine set_it(npara,ivalue,value,name,id,
c      subroutine case_trap(string,length)
c      subroutine no_spaces(buff,nchars)
c---------------------------------------------------------------------- 


      subroutine setpara(param_name,readlha)
c***********************************************************************
c This subroutine sets up the HELAS couplings of the STANDARD MODEL.
c***********************************************************************
      implicit none
c
c local
c
      character*(*) param_name
      logical readlha
      integer i
      real*8 dum
c
c     common file with the couplings
c
      include 'coupl.inc'
      include 'input.inc'
c
c     local
c
      double precision  v
      double precision  ee, ee2, ez, ey, sw, cw, sc2, sin2w, wm
      double precision  gwne, gwud, lambda, lam4, xt, rew, rqcd
      double precision  alphas, alfa, alfaw, mfrun
      external          alphas, alfa, alfaw, mfrun
c
c     Common to lh_readin and printout
c
      double precision  alpha, gfermi, alfas
      double precision  mtMS,mbMS,mcMS,mtaMS!MSbar masses
      double precision  Vud,Vus             !CKM matrix elements
      common/values/    alpha,gfermi,alfas,   
     &                  mtMS,mbMS,mcMS,mtaMS,
     &                  Vud
c
c constants
c
      double complex  ci
      parameter( ci = ( 0.0d0, 1.0d0 ) )
      double precision  Zero, One, Two, Three, Four, Half, Rt2
      parameter( Zero = 0.0d0, One = 1.0d0, Two = 2.0d0 )
      parameter( Three = 3.0d0, Four = 4.0d0, Half = 0.5d0 )
      parameter( Rt2   = 1.414213562d0 )
      double precision  Pi, Fourpi
      parameter( Pi = 3.14159265358979323846d0 )
      parameter( Fourpi = Four * Pi )
c
c     alfas and run
c************
c Uncomment the following lines in order to use alphas from the PDF
c      include '../alfas.inc'
c      include '../run.inc'
c***********
c------------------------------------------
c Start calculating the couplings for HELAS
c------------------------------------------
c
      if(readlha) then 
         call lh_readin(param_name)
         G = DSQRT(4d0*PI*ALFAS) ! use setting of the param_card.dat @ NLO
      endif
c     

      GG(1) = -G
      GG(2) = -G     
c
c auxiliary local values
c
      wm = sqrt(zmass**2/Two+
     $     sqrt(zmass**4/Four-Pi/Rt2*alpha/gfermi*zmass**2))
      sin2w  = One-(wm/zmass)**2
      cw  = sqrt( One - sin2w )
      ee2 = alpha * Fourpi
      sw  = sqrt( sin2w )
      ee  = sqrt( ee2 )
      ez  = ee/(sw*cw)
      ey  = ee*(sw/cw)
      sc2 = sin2w*( One - sin2w )
      v   = Two*wm*sw/ee   ! the wmass is used to calculate v
      lambda = hmass**2 / (Two * v**2)
c
c vector boson couplings
c
      gw   = ee/sw
      gwwa = ee
      gwwz = ee*cw/sw
c
c gauge & higgs boson coupling constants
c
      gwwh  = dcmplx( ee2/sin2w*Half*v, Zero )
      gzzh  = dcmplx( ee2/sc2*Half*v, Zero )
      ghhh  = dcmplx( -hmass**2/v*Three, Zero )
      gwwhh = dcmplx( ee2/sin2w*Half, Zero )
      gzzhh = dcmplx( ee2/sc2*Half, Zero)
      ghhhh = ghhh/v
c
c fermion-fermion-vector couplings
c
      gal(1) = dcmplx(  ee          , Zero )
      gal(2) = dcmplx(  ee          , Zero )
      gau(1) = dcmplx( -ee*Two/Three, Zero )
      gau(2) = dcmplx( -ee*Two/Three, Zero )
      gad(1) = dcmplx(  ee/Three    , Zero )
      gad(2) = dcmplx(  ee/Three    , Zero )

      gwf(1) = dcmplx( -ee/sqrt(Two*sin2w), Zero )
      gwf(2) = dcmplx(  Zero              , Zero )

      gzn(1) = dcmplx( -ez*Half                     , Zero )
      gzn(2) = dcmplx(  Zero                        , Zero )
      gzl(1) = dcmplx( -ez*(-Half + sin2w)          , Zero )
      gzl(2) = dcmplx( -ey                          , Zero )
      gzu(1) = dcmplx( -ez*( Half - sin2w*Two/Three), Zero )
      gzu(2) = dcmplx(  ey*Two/Three                , Zero )
      gzd(1) = dcmplx( -ez*(-Half + sin2w/Three)    , Zero )
      gzd(2) = dcmplx( -ey/Three                    , Zero )

c fermion-fermion-Higgs couplings (complex) hff(2)
c
c NOTE: the running mass is evaluated @ the same order 
c nloop of alpha_s set by the PDF choice
c 

      if(mtMS.gt.1d0) then
         ghtop(1) = dcmplx( -mtMS/v, Zero )
      else
         ghtop(1) = dcmplx( Zero,Zero)
      endif
      ghtop(2) = ghtop(1)

      if(mbMS.gt.1d0) then
         ghbot(1) = dcmplx( -mbMS/v, Zero )
      else
         ghbot(1) = dcmplx( Zero, Zero )
      endif
      ghbot(2) = ghbot(1)
      
      if(mcMS.gt.1d0) then
         ghcha(1) = dcmplx( -mcMS/v, Zero )
      else
         ghcha(1) = dcmplx( Zero, Zero )
      endif
      ghcha(2) = ghcha(1)

      ghtau(1) = dcmplx( -mtaMS/v, Zero )
      ghtau(2) = ghtau(1)

c
c     CKM matrix: 
c     symmetric 3x3 matrix, Vud=Vcs, Vus=Vcd Vcb=Vub=0
c
c     >>>>>>>>>>>>>>>***** NOTE****<<<<<<<<<<<<<<<<<<<<<<<<<
c     these couplings matter only when interaction_CKM.dat
c     is used to generate all the diagrams with off-diagonal
c     couplings. The default of MadEvent is a diagonal
c     CKM matrix.

	  Vus=DSQRT(1d0-Vud**2)
      do i=1,2
         gwfc(i) = gwf(i)*Vud
         gwfs(i) = gwf(i)*Vus
         gwfm(i) =-gwf(i)*Vus
      enddo

c---------------------------------------------------------
c Set Photon Width to Zero, used by symmetry optimization
c---------------------------------------------------------

      awidth = 0d0
c************************************            
c UserMode couplings
c************************************

      GSLQ = dcmplx(-G)
      G2SLQ = dcmplx(G**2)


c **** R2 ****
      G1UL11(1) = -dcmplx(y2RL_11_r, y2RL_11_i)
      G1UL11(2) =  conjg(dcmplx(y2LR_11_r, y2LR_11_i))
      G1UL21(1) = -dcmplx(y2RL_21_r, y2RL_21_i)
      G1UL21(2) =  conjg(dcmplx(y2LR_21_r, y2LR_21_i))
      G1UL31(1) = -dcmplx(y2RL_31_r, y2RL_31_i)
      G1UL31(2) =  conjg(dcmplx(y2LR_31_r, y2LR_31_i))
      G1UL12(1) = -dcmplx(y2RL_12_r, y2RL_12_i)
      G1UL12(2) =  conjg(dcmplx(y2LR_12_r, y2LR_12_i))
      G1UL22(1) = -dcmplx(y2RL_22_r, y2RL_22_i)
      G1UL22(2) =  conjg(dcmplx(y2LR_22_r, y2LR_22_i))
      G1UL32(1) = -dcmplx(y2RL_32_r, y2RL_32_i)
      G1UL32(2) =  conjg(dcmplx(y2LR_32_r, y2LR_32_i))
      G1UL13(1) = -dcmplx(y2RL_13_r, y2RL_13_i)
      G1UL13(2) =  conjg(dcmplx(y2LR_13_r, y2LR_13_i))
      G1UL23(1) = -dcmplx(y2RL_23_r, y2RL_23_i)
      G1UL23(2) =  conjg(dcmplx(y2LR_23_r, y2LR_23_i))
      G1UL33(1) = -dcmplx(y2RL_33_r, y2RL_33_i)
      G1UL33(2) =  conjg(dcmplx(y2LR_33_r, y2LR_33_i))

      G2UN11(1) = dcmplx(y2RL_11_r, y2RL_11_i)
      G2UN11(2) = dcmplx(Zero)
      G2UN21(1) = dcmplx(y2RL_21_r, y2RL_21_i)
      G2UN21(2) = dcmplx(Zero)
      G2UN31(1) = dcmplx(y2RL_31_r, y2RL_31_i)
      G2UN31(2) = dcmplx(Zero)
      G2UN12(1) = dcmplx(y2RL_12_r, y2RL_12_i)
      G2UN12(2) = dcmplx(Zero)
      G2UN22(1) = dcmplx(y2RL_22_r, y2RL_22_i)
      G2UN22(2) = dcmplx(Zero)
      G2UN32(1) = dcmplx(y2RL_32_r, y2RL_32_i)
      G2UN32(2) = dcmplx(Zero)
      G2UN13(1) = dcmplx(y2RL_13_r, y2RL_13_i)
      G2UN13(2) = dcmplx(Zero)
      G2UN23(1) = dcmplx(y2RL_23_r, y2RL_23_i)
      G2UN23(2) = dcmplx(Zero)
      G2UN33(1) = dcmplx(y2RL_33_r, y2RL_33_i)
      G2UN33(2) = dcmplx(Zero)

      G2DL11(1) = dcmplx(Zero)
      G2DL11(2) = conjg(dcmplx(y2LR_11_r, y2LR_11_i))
      G2DL21(1) = dcmplx(Zero)
      G2DL21(2) = conjg(dcmplx(y2LR_21_r, y2LR_21_i))
      G2DL31(1) = dcmplx(Zero)
      G2DL31(2) = conjg(dcmplx(y2LR_31_r, y2LR_31_i))
      G2DL12(1) = dcmplx(Zero)
      G2DL12(2) = conjg(dcmplx(y2LR_12_r, y2LR_12_i))
      G2DL22(1) = dcmplx(Zero)
      G2DL22(2) = conjg(dcmplx(y2LR_22_r, y2LR_22_i))
      G2DL32(1) = dcmplx(Zero)
      G2DL32(2) = conjg(dcmplx(y2LR_32_r, y2LR_32_i))
      G2DL13(1) = dcmplx(Zero)
      G2DL13(2) = conjg(dcmplx(y2LR_13_r, y2LR_13_i))
      G2DL23(1) = dcmplx(Zero)
      G2DL23(2) = conjg(dcmplx(y2LR_23_r, y2LR_23_i))
      G2DL33(1) = dcmplx(Zero)
      G2DL33(2) = conjg(dcmplx(y2LR_33_r, y2LR_33_i))

      G1UL11C(1) = conjg(G1UL11(2))
      G1UL11C(2) = conjg(G1UL11(1))
      G1UL21C(1) = conjg(G1UL21(2))
      G1UL21C(2) = conjg(G1UL21(1))
      G1UL31C(1) = conjg(G1UL31(2))
      G1UL31C(2) = conjg(G1UL31(1))
      G1UL12C(1) = conjg(G1UL12(2))
      G1UL12C(2) = conjg(G1UL12(1))
      G1UL22C(1) = conjg(G1UL22(2))
      G1UL22C(2) = conjg(G1UL22(1))
      G1UL32C(1) = conjg(G1UL32(2))
      G1UL32C(2) = conjg(G1UL32(1))
      G1UL13C(1) = conjg(G1UL13(2))
      G1UL13C(2) = conjg(G1UL13(1))
      G1UL23C(1) = conjg(G1UL23(2))
      G1UL23C(2) = conjg(G1UL23(1))
      G1UL33C(1) = conjg(G1UL33(2))
      G1UL33C(2) = conjg(G1UL33(1))

      G2UN11C(1) = conjg(G2UN11(2))
      G2UN11C(2) = conjg(G2UN11(1))
      G2UN21C(1) = conjg(G2UN21(2))
      G2UN21C(2) = conjg(G2UN21(1))
      G2UN31C(1) = conjg(G2UN31(2))
      G2UN31C(2) = conjg(G2UN31(1))
      G2UN12C(1) = conjg(G2UN12(2))
      G2UN12C(2) = conjg(G2UN12(1))
      G2UN22C(1) = conjg(G2UN22(2))
      G2UN22C(2) = conjg(G2UN22(1))
      G2UN32C(1) = conjg(G2UN32(2))
      G2UN32C(2) = conjg(G2UN32(1))
      G2UN13C(1) = conjg(G2UN13(2))
      G2UN13C(2) = conjg(G2UN13(1))
      G2UN23C(1) = conjg(G2UN23(2))
      G2UN23C(2) = conjg(G2UN23(1))
      G2UN33C(1) = conjg(G2UN33(2))
      G2UN33C(2) = conjg(G2UN33(1))

      G2DL11C(1) = conjg(G2DL11(2))
      G2DL11C(2) = conjg(G2DL11(1))
      G2DL21C(1) = conjg(G2DL21(2))
      G2DL21C(2) = conjg(G2DL21(1))
      G2DL31C(1) = conjg(G2DL31(2))
      G2DL31C(2) = conjg(G2DL31(1))
      G2DL12C(1) = conjg(G2DL12(2))
      G2DL12C(2) = conjg(G2DL12(1))
      G2DL22C(1) = conjg(G2DL22(2))
      G2DL22C(2) = conjg(G2DL22(1))
      G2DL32C(1) = conjg(G2DL32(2))
      G2DL32C(2) = conjg(G2DL32(1))
      G2DL13C(1) = conjg(G2DL13(2))
      G2DL13C(2) = conjg(G2DL13(1))
      G2DL23C(1) = conjg(G2DL23(2))
      G2DL23C(2) = conjg(G2DL23(1))
      G2DL33C(1) = conjg(G2DL33(2))
      G2DL33C(2) = conjg(G2DL33(1))


c **** S1 ****
      G3UL11(1) = dcmplx(y1LL_11_r, y1LL_11_i)
      G3UL11(2) = dcmplx(y1RR_11_r, y1RR_11_i)
      G3UL21(1) = dcmplx(y1LL_21_r, y1LL_21_i)
      G3UL21(2) = dcmplx(y1RR_21_r, y1RR_21_i)
      G3UL31(1) = dcmplx(y1LL_31_r, y1LL_31_i)
      G3UL31(2) = dcmplx(y1RR_31_r, y1RR_31_i)
      G3UL12(1) = dcmplx(y1LL_12_r, y1LL_12_i)
      G3UL12(2) = dcmplx(y1RR_12_r, y1RR_12_i)
      G3UL22(1) = dcmplx(y1LL_22_r, y1LL_22_i)
      G3UL22(2) = dcmplx(y1RR_22_r, y1RR_22_i)
      G3UL32(1) = dcmplx(y1LL_32_r, y1LL_32_i)
      G3UL32(2) = dcmplx(y1RR_32_r, y1RR_32_i)
      G3UL13(1) = dcmplx(y1LL_13_r, y1LL_13_i)
      G3UL13(2) = dcmplx(y1RR_13_r, y1RR_13_i)
      G3UL23(1) = dcmplx(y1LL_23_r, y1LL_23_i)
      G3UL23(2) = dcmplx(y1RR_23_r, y1RR_23_i)
      G3UL33(1) = dcmplx(y1LL_33_r, y1LL_33_i)
      G3UL33(2) = dcmplx(y1RR_33_r, y1RR_33_i)

      G3DN11(1) = dcmplx(-y1LL_11_r, -y1LL_11_i)
      G3DN11(2) = dcmplx(Zero)
      G3DN21(1) = dcmplx(-y1LL_21_r, -y1LL_21_i)
      G3DN21(2) = dcmplx(Zero)
      G3DN31(1) = dcmplx(-y1LL_31_r, -y1LL_31_i)
      G3DN31(2) = dcmplx(Zero)
      G3DN12(1) = dcmplx(-y1LL_12_r, -y1LL_12_i)
      G3DN12(2) = dcmplx(Zero)
      G3DN22(1) = dcmplx(-y1LL_22_r, -y1LL_22_i)
      G3DN22(2) = dcmplx(Zero)
      G3DN32(1) = dcmplx(-y1LL_32_r, -y1LL_32_i)
      G3DN32(2) = dcmplx(Zero)
      G3DN13(1) = dcmplx(-y1LL_13_r, -y1LL_13_i)
      G3DN13(2) = dcmplx(Zero)
      G3DN23(1) = dcmplx(-y1LL_23_r, -y1LL_23_i)
      G3DN23(2) = dcmplx(Zero)
      G3DN33(1) = dcmplx(-y1LL_33_r, -y1LL_33_i)
      G3DN33(2) = dcmplx(Zero)

      G3UL11C(1) = conjg(G3UL11(2))
      G3UL11C(2) = conjg(G3UL11(1))
      G3UL21C(1) = conjg(G3UL21(2))
      G3UL21C(2) = conjg(G3UL21(1))
      G3UL31C(1) = conjg(G3UL31(2))
      G3UL31C(2) = conjg(G3UL31(1))
      G3UL12C(1) = conjg(G3UL12(2))
      G3UL12C(2) = conjg(G3UL12(1))
      G3UL22C(1) = conjg(G3UL22(2))
      G3UL22C(2) = conjg(G3UL22(1))
      G3UL32C(1) = conjg(G3UL32(2))
      G3UL32C(2) = conjg(G3UL32(1))
      G3UL13C(1) = conjg(G3UL13(2))
      G3UL13C(2) = conjg(G3UL13(1))
      G3UL23C(1) = conjg(G3UL23(2))
      G3UL23C(2) = conjg(G3UL23(1))
      G3UL33C(1) = conjg(G3UL33(2))
      G3UL33C(2) = conjg(G3UL33(1))

      G3DN11C(1) = conjg(G3DN11(2))
      G3DN11C(2) = conjg(G3DN11(1))
      G3DN21C(1) = conjg(G3DN21(2))
      G3DN21C(2) = conjg(G3DN21(1))
      G3DN31C(1) = conjg(G3DN31(2))
      G3DN31C(2) = conjg(G3DN31(1))
      G3DN12C(1) = conjg(G3DN12(2))
      G3DN12C(2) = conjg(G3DN12(1))
      G3DN22C(1) = conjg(G3DN22(2))
      G3DN22C(2) = conjg(G3DN22(1))
      G3DN32C(1) = conjg(G3DN32(2))
      G3DN32C(2) = conjg(G3DN32(1))
      G3DN13C(1) = conjg(G3DN13(2))
      G3DN13C(2) = conjg(G3DN13(1))
      G3DN23C(1) = conjg(G3DN23(2))
      G3DN23C(2) = conjg(G3DN23(1))
      G3DN33C(1) = conjg(G3DN33(2))
      G3DN33C(2) = conjg(G3DN33(1))


c **** S3 ****
      G4UN11(1) = dsqrt(2d0)*dcmplx(y3LL_11_r, y3LL_11_i)
      G4UN11(2) = dcmplx(Zero)
      G4UN21(1) = dsqrt(2d0)*dcmplx(y3LL_21_r, y3LL_21_i)
      G4UN21(2) = dcmplx(Zero)
      G4UN31(1) = dsqrt(2d0)*dcmplx(y3LL_31_r, y3LL_31_i)
      G4UN31(2) = dcmplx(Zero)
      G4UN12(1) = dsqrt(2d0)*dcmplx(y3LL_12_r, y3LL_12_i)
      G4UN12(2) = dcmplx(Zero)
      G4UN22(1) = dsqrt(2d0)*dcmplx(y3LL_22_r, y3LL_22_i)
      G4UN22(2) = dcmplx(Zero)
      G4UN32(1) = dsqrt(2d0)*dcmplx(y3LL_32_r, y3LL_32_i)
      G4UN32(2) = dcmplx(Zero)
      G4UN13(1) = dsqrt(2d0)*dcmplx(y3LL_13_r, y3LL_13_i)
      G4UN13(2) = dcmplx(Zero)
      G4UN23(1) = dsqrt(2d0)*dcmplx(y3LL_23_r, y3LL_23_i)
      G4UN23(2) = dcmplx(Zero)
      G4UN33(1) = dsqrt(2d0)*dcmplx(y3LL_33_r, y3LL_33_i)
      G4UN33(2) = dcmplx(Zero)

      G5UL11(1) = -dcmplx(y3LL_11_r, y3LL_11_i)
      G5UL11(2) = dcmplx(Zero)
      G5UL21(1) = -dcmplx(y3LL_21_r, y3LL_21_i)
      G5UL21(2) = dcmplx(Zero)
      G5UL31(1) = -dcmplx(y3LL_31_r, y3LL_31_i)
      G5UL31(2) = dcmplx(Zero)
      G5UL12(1) = -dcmplx(y3LL_12_r, y3LL_12_i)
      G5UL12(2) = dcmplx(Zero)
      G5UL22(1) = -dcmplx(y3LL_22_r, y3LL_22_i)
      G5UL22(2) = dcmplx(Zero)
      G5UL32(1) = -dcmplx(y3LL_32_r, y3LL_32_i)
      G5UL32(2) = dcmplx(Zero)
      G5UL13(1) = -dcmplx(y3LL_13_r, y3LL_13_i)
      G5UL13(2) = dcmplx(Zero)
      G5UL23(1) = -dcmplx(y3LL_23_r, y3LL_23_i)
      G5UL23(2) = dcmplx(Zero)
      G5UL33(1) = -dcmplx(y3LL_33_r, y3LL_33_i)
      G5UL33(2) = dcmplx(Zero)

      G5DN11(1) = -dcmplx(y3LL_11_r, y3LL_11_i)
      G5DN11(2) = dcmplx(Zero)
      G5DN21(1) = -dcmplx(y3LL_21_r, y3LL_21_i)
      G5DN21(2) = dcmplx(Zero)
      G5DN31(1) = -dcmplx(y3LL_31_r, y3LL_31_i)
      G5DN31(2) = dcmplx(Zero)
      G5DN12(1) = -dcmplx(y3LL_12_r, y3LL_12_i)
      G5DN12(2) = dcmplx(Zero)
      G5DN22(1) = -dcmplx(y3LL_22_r, y3LL_22_i)
      G5DN22(2) = dcmplx(Zero)
      G5DN32(1) = -dcmplx(y3LL_32_r, y3LL_32_i)
      G5DN32(2) = dcmplx(Zero)
      G5DN13(1) = -dcmplx(y3LL_13_r, y3LL_13_i)
      G5DN13(2) = dcmplx(Zero)
      G5DN23(1) = -dcmplx(y3LL_23_r, y3LL_23_i)
      G5DN23(2) = dcmplx(Zero)
      G5DN33(1) = -dcmplx(y3LL_33_r, y3LL_33_i)
      G5DN33(2) = dcmplx(Zero)

      G6DL11(1) = -dsqrt(2d0)*dcmplx(y3LL_11_r, y3LL_11_i)
      G6DL11(2) = dcmplx(Zero)
      G6DL21(1) = -dsqrt(2d0)*dcmplx(y3LL_21_r, y3LL_21_i)
      G6DL21(2) = dcmplx(Zero)
      G6DL31(1) = -dsqrt(2d0)*dcmplx(y3LL_31_r, y3LL_31_i)
      G6DL31(2) = dcmplx(Zero)
      G6DL12(1) = -dsqrt(2d0)*dcmplx(y3LL_12_r, y3LL_12_i)
      G6DL12(2) = dcmplx(Zero)
      G6DL22(1) = -dsqrt(2d0)*dcmplx(y3LL_22_r, y3LL_22_i)
      G6DL22(2) = dcmplx(Zero)
      G6DL32(1) = -dsqrt(2d0)*dcmplx(y3LL_32_r, y3LL_32_i)
      G6DL32(2) = dcmplx(Zero)
      G6DL13(1) = -dsqrt(2d0)*dcmplx(y3LL_13_r, y3LL_13_i)
      G6DL13(2) = dcmplx(Zero)
      G6DL23(1) = -dsqrt(2d0)*dcmplx(y3LL_23_r, y3LL_23_i)
      G6DL23(2) = dcmplx(Zero)
      G6DL33(1) = -dsqrt(2d0)*dcmplx(y3LL_33_r, y3LL_33_i)
      G6DL33(2) = dcmplx(Zero)

      G4UN11C(1) = conjg(G4UN11(2))
      G4UN11C(2) = conjg(G4UN11(1))
      G4UN21C(1) = conjg(G4UN21(2))
      G4UN21C(2) = conjg(G4UN21(1))
      G4UN31C(1) = conjg(G4UN31(2))
      G4UN31C(2) = conjg(G4UN31(1))
      G4UN12C(1) = conjg(G4UN12(2))
      G4UN12C(2) = conjg(G4UN12(1))
      G4UN22C(1) = conjg(G4UN22(2))
      G4UN22C(2) = conjg(G4UN22(1))
      G4UN32C(1) = conjg(G4UN32(2))
      G4UN32C(2) = conjg(G4UN32(1))
      G4UN13C(1) = conjg(G4UN13(2))
      G4UN13C(2) = conjg(G4UN13(1))
      G4UN23C(1) = conjg(G4UN23(2))
      G4UN23C(2) = conjg(G4UN23(1))
      G4UN33C(1) = conjg(G4UN33(2))
      G4UN33C(2) = conjg(G4UN33(1))

      G5UL11C(1) = conjg(G5UL11(2))
      G5UL11C(2) = conjg(G5UL11(1))
      G5UL21C(1) = conjg(G5UL21(2))
      G5UL21C(2) = conjg(G5UL21(1))
      G5UL31C(1) = conjg(G5UL31(2))
      G5UL31C(2) = conjg(G5UL31(1))
      G5UL12C(1) = conjg(G5UL12(2))
      G5UL12C(2) = conjg(G5UL12(1))
      G5UL22C(1) = conjg(G5UL22(2))
      G5UL22C(2) = conjg(G5UL22(1))
      G5UL32C(1) = conjg(G5UL32(2))
      G5UL32C(2) = conjg(G5UL32(1))
      G5UL13C(1) = conjg(G5UL13(2))
      G5UL13C(2) = conjg(G5UL13(1))
      G5UL23C(1) = conjg(G5UL23(2))
      G5UL23C(2) = conjg(G5UL23(1))
      G5UL33C(1) = conjg(G5UL33(2))
      G5UL33C(2) = conjg(G5UL33(1))

      G5DN11C(1) = conjg(G5DN11(2))
      G5DN11C(2) = conjg(G5DN11(1))
      G5DN21C(1) = conjg(G5DN21(2))
      G5DN21C(2) = conjg(G5DN21(1))
      G5DN31C(1) = conjg(G5DN31(2))
      G5DN31C(2) = conjg(G5DN31(1))
      G5DN12C(1) = conjg(G5DN12(2))
      G5DN12C(2) = conjg(G5DN12(1))
      G5DN22C(1) = conjg(G5DN22(2))
      G5DN22C(2) = conjg(G5DN22(1))
      G5DN32C(1) = conjg(G5DN32(2))
      G5DN32C(2) = conjg(G5DN32(1))
      G5DN13C(1) = conjg(G5DN13(2))
      G5DN13C(2) = conjg(G5DN13(1))
      G5DN23C(1) = conjg(G5DN23(2))
      G5DN23C(2) = conjg(G5DN23(1))
      G5DN33C(1) = conjg(G5DN33(2))
      G5DN33C(2) = conjg(G5DN33(1))

      G6DL11C(1) = conjg(G6DL11(2))
      G6DL11C(2) = conjg(G6DL11(1))
      G6DL21C(1) = conjg(G6DL21(2))
      G6DL21C(2) = conjg(G6DL21(1))
      G6DL31C(1) = conjg(G6DL31(2))
      G6DL31C(2) = conjg(G6DL31(1))
      G6DL12C(1) = conjg(G6DL12(2))
      G6DL12C(2) = conjg(G6DL12(1))
      G6DL22C(1) = conjg(G6DL22(2))
      G6DL22C(2) = conjg(G6DL22(1))
      G6DL32C(1) = conjg(G6DL32(2))
      G6DL32C(2) = conjg(G6DL32(1))
      G6DL13C(1) = conjg(G6DL13(2))
      G6DL13C(2) = conjg(G6DL13(1))
      G6DL23C(1) = conjg(G6DL23(2))
      G6DL23C(2) = conjg(G6DL23(1))
      G6DL33C(1) = conjg(G6DL33(2))
      G6DL33C(2) = conjg(G6DL33(1))


c **** R2tilde ****
      G7DL11(1) = -dcmplx(y2tildeRL_11_r, y2tildeRL_11_i)
      G7DL11(2) = dcmplx(Zero)
      G7DL21(1) = -dcmplx(y2tildeRL_21_r, y2tildeRL_21_i)
      G7DL21(2) = dcmplx(Zero)
      G7DL31(1) = -dcmplx(y2tildeRL_31_r, y2tildeRL_31_i)
      G7DL31(2) = dcmplx(Zero)
      G7DL12(1) = -dcmplx(y2tildeRL_12_r, y2tildeRL_12_i)
      G7DL12(2) = dcmplx(Zero)
      G7DL22(1) = -dcmplx(y2tildeRL_22_r, y2tildeRL_22_i)
      G7DL22(2) = dcmplx(Zero)
      G7DL32(1) = -dcmplx(y2tildeRL_32_r, y2tildeRL_32_i)
      G7DL32(2) = dcmplx(Zero)
      G7DL13(1) = -dcmplx(y2tildeRL_13_r, y2tildeRL_13_i)
      G7DL13(2) = dcmplx(Zero)
      G7DL23(1) = -dcmplx(y2tildeRL_23_r, y2tildeRL_23_i)
      G7DL23(2) = dcmplx(Zero)
      G7DL33(1) = -dcmplx(y2tildeRL_33_r, y2tildeRL_33_i)
      G7DL33(2) = dcmplx(Zero)

      G8DN11(1) = dcmplx(y2tildeRL_11_r, y2tildeRL_11_i)
      G8DN11(2) = dcmplx(Zero)
      G8DN21(1) = dcmplx(y2tildeRL_21_r, y2tildeRL_21_i)
      G8DN21(2) = dcmplx(Zero)
      G8DN31(1) = dcmplx(y2tildeRL_31_r, y2tildeRL_31_i)
      G8DN31(2) = dcmplx(Zero)
      G8DN12(1) = dcmplx(y2tildeRL_12_r, y2tildeRL_12_i)
      G8DN12(2) = dcmplx(Zero)
      G8DN22(1) = dcmplx(y2tildeRL_22_r, y2tildeRL_22_i)
      G8DN22(2) = dcmplx(Zero)
      G8DN32(1) = dcmplx(y2tildeRL_32_r, y2tildeRL_32_i)
      G8DN32(2) = dcmplx(Zero)
      G8DN13(1) = dcmplx(y2tildeRL_13_r, y2tildeRL_13_i)
      G8DN13(2) = dcmplx(Zero)
      G8DN23(1) = dcmplx(y2tildeRL_23_r, y2tildeRL_23_i)
      G8DN23(2) = dcmplx(Zero)
      G8DN33(1) = dcmplx(y2tildeRL_33_r, y2tildeRL_33_i)
      G8DN33(2) = dcmplx(Zero)

      G7DL11C(1) = conjg(G7DL11(2))
      G7DL11C(2) = conjg(G7DL11(1))
      G7DL21C(1) = conjg(G7DL21(2))
      G7DL21C(2) = conjg(G7DL21(1))
      G7DL31C(1) = conjg(G7DL31(2))
      G7DL31C(2) = conjg(G7DL31(1))
      G7DL12C(1) = conjg(G7DL12(2))
      G7DL12C(2) = conjg(G7DL12(1))
      G7DL22C(1) = conjg(G7DL22(2))
      G7DL22C(2) = conjg(G7DL22(1))
      G7DL32C(1) = conjg(G7DL32(2))
      G7DL32C(2) = conjg(G7DL32(1))
      G7DL13C(1) = conjg(G7DL13(2))
      G7DL13C(2) = conjg(G7DL13(1))
      G7DL23C(1) = conjg(G7DL23(2))
      G7DL23C(2) = conjg(G7DL23(1))
      G7DL33C(1) = conjg(G7DL33(2))
      G7DL33C(2) = conjg(G7DL33(1))

      G8DN11C(1) = conjg(G8DN11(2))
      G8DN11C(2) = conjg(G8DN11(1))
      G8DN21C(1) = conjg(G8DN21(2))
      G8DN21C(2) = conjg(G8DN21(1))
      G8DN31C(1) = conjg(G8DN31(2))
      G8DN31C(2) = conjg(G8DN31(1))
      G8DN12C(1) = conjg(G8DN12(2))
      G8DN12C(2) = conjg(G8DN12(1))
      G8DN22C(1) = conjg(G8DN22(2))
      G8DN22C(2) = conjg(G8DN22(1))
      G8DN32C(1) = conjg(G8DN32(2))
      G8DN32C(2) = conjg(G8DN32(1))
      G8DN13C(1) = conjg(G8DN13(2))
      G8DN13C(2) = conjg(G8DN13(1))
      G8DN23C(1) = conjg(G8DN23(2))
      G8DN23C(2) = conjg(G8DN23(1))
      G8DN33C(1) = conjg(G8DN33(2))
      G8DN33C(2) = conjg(G8DN33(1))


c **** S1tilde ****
      G9DL11(1) = dcmplx(Zero)
      G9DL11(2) = dcmplx(y1tildeRR_11_r, y1tildeRR_11_i)
      G9DL21(1) = dcmplx(Zero)
      G9DL21(2) = dcmplx(y1tildeRR_21_r, y1tildeRR_21_i)
      G9DL31(1) = dcmplx(Zero)
      G9DL31(2) = dcmplx(y1tildeRR_31_r, y1tildeRR_31_i)
      G9DL12(1) = dcmplx(Zero)
      G9DL12(2) = dcmplx(y1tildeRR_12_r, y1tildeRR_12_i)
      G9DL22(1) = dcmplx(Zero)
      G9DL22(2) = dcmplx(y1tildeRR_22_r, y1tildeRR_22_i)
      G9DL32(1) = dcmplx(Zero)
      G9DL32(2) = dcmplx(y1tildeRR_32_r, y1tildeRR_32_i)
      G9DL13(1) = dcmplx(Zero)
      G9DL13(2) = dcmplx(y1tildeRR_13_r, y1tildeRR_13_i)
      G9DL23(1) = dcmplx(Zero)
      G9DL23(2) = dcmplx(y1tildeRR_23_r, y1tildeRR_23_i)
      G9DL33(1) = dcmplx(Zero)
      G9DL33(2) = dcmplx(y1tildeRR_33_r, y1tildeRR_33_i)

      G9DL11C(1) = conjg(G9DL11(2))
      G9DL11C(2) = conjg(G9DL11(1))
      G9DL21C(1) = conjg(G9DL21(2))
      G9DL21C(2) = conjg(G9DL21(1))
      G9DL31C(1) = conjg(G9DL31(2))
      G9DL31C(2) = conjg(G9DL31(1))
      G9DL12C(1) = conjg(G9DL12(2))
      G9DL12C(2) = conjg(G9DL12(1))
      G9DL22C(1) = conjg(G9DL22(2))
      G9DL22C(2) = conjg(G9DL22(1))
      G9DL32C(1) = conjg(G9DL32(2))
      G9DL32C(2) = conjg(G9DL32(1))
      G9DL13C(1) = conjg(G9DL13(2))
      G9DL13C(2) = conjg(G9DL13(1))
      G9DL23C(1) = conjg(G9DL23(2))
      G9DL23C(2) = conjg(G9DL23(1))
      G9DL33C(1) = conjg(G9DL33(2))
      G9DL33C(2) = conjg(G9DL33(1))


c----------------------------
c end subroutine coupsm
c----------------------------


      return
      end

      
